/*********************************************************************************
* This file is a part of SATFO                                                   *
* it is created to optimize fractions of concurent transitions in                *
* a Petri net model of genistein elimination pathway                             *
* for details see publication                                                    *
*                                                                                *
* P. Reshetova, A. K. Smilde, J. A. Westerhuis, A. H. C. van Kampen              *
* "Modeling and studying multi-organ elimination pathways using Petri nets"      *
*                                                                                *
* created by Polina Reshetova, 2014                                              *
*********************************************************************************/


#include <stdio.h>
#include <math.h>
#include <string.h>

#include "calloc.h"
#include "SArandom.h"

#define SQR(a)            ((a)*(a))

//do not forget to change  SATFOv40.set. number of parameters 

double EvaluateCostFunction (int, double *);
void   GenerateNewParametersUser(double *, double *,double *,
				 double *, int, double);
void   Main                 (int, char **);



void main (int argc, char *argv[]) 
{
 SA(argc, argv);
}


double EvaluateCostFunction (int     NumberOfParameters,
			     double *ParamValues)

{
 int i,j;
 double Error;

 Error=0.0;

//run Petri nets simulation program
char stream[500];
int stream_i = 0;

char command[] = "./PN/foranealen -t \"R1:R7:R2:R30:R31:R11:R3:R5:R8:R17:R16:R13:R20:R18:R15:R26:R25:R22:R29:R24:R27\" -p \"";

strcpy(stream, command);

char buffer[3];
sprintf(buffer, "%lf", ParamValues[0]);

strcat(stream, buffer);

for (i=1;i<NumberOfParameters;i++)
{
  sprintf(buffer, ":%lf", ParamValues[i]);
  strcat(stream,buffer);
}

strcat(stream,"\"");


int number_of_places = 3;
//repeat Petri nets simulation number_of_reps times and calculate the mean for each profile
//to minimize the stochastic effect of Petri nets simulation
int number_of_reps = 20;

int set_new_profile[number_of_reps][number_of_places][7];
int g;
for(g=0;g<number_of_reps;g++){

  //if there is an error, print it
  int ret = system(stream);
  if(ret != 0) 
  {
    printf("somthing is wrong\n");
    printf("command was %s\n",stream);
    exit(-1);
  }

  //read the new profile
  const char profileFileName[] = "./stepProfile.txt";
  FILE *fileHandle = fopen (profileFileName, "r");
   
  if (!fileHandle)
  {
    fprintf(stderr, "Error - Cannot open %s for reading!\n", 
    profileFileName);
      
    exit (-1);
  }

  char line[1024] = {0x0};
  char *pEnd, *pch, *linep;
  int num;

  for(i=0;i<7;i++)
  {
    fgets(line,1024,fileHandle);  

    for (j = 0, linep = line; (pch = strtok(linep, "\t")) != NULL;j++, linep = NULL)
    {    
      // fprintf(stderr,"string is %s\n", pch);
      num = (int) strtol(pch, &pEnd, 10);
    
      set_new_profile[g][j][i] = num;           
    }  
  }

  fclose(fileHandle);
}

double new_profile[number_of_places][7];
//calculate the mean profiles for number_of_reps runs
for(i=0;i<7;i++)
{
  for(j=0;j<number_of_places;j++)
  {
    int sum_over_ten = 0;
    for (g=0;g<number_of_reps;g++)
    {
      sum_over_ten += set_new_profile[g][j][i];
    }
    new_profile[j][i] = (double)sum_over_ten / (double)number_of_reps;
  }
}

//reference profile (1000 input tokens number)
//only three determinant places are used
int old_profile[3][7] = 
{{0,154,189,157,53,10,0},
{0,14,29,50,36,16,0},{0,1,7,28,42,28,0},};

//calculate error
for(j=0; j<number_of_places; j++)
{
   for(i=0; i<7; i++) 
   {
     Error += sqrt((double) SQR(( new_profile[j][i]- old_profile[j][i] )));
   }
}

//fprintf(stderr,"error is %f\n", Error);
return(Error);
 
}


void GenerateNewParametersUser (double *ParametersNew, 
				double *Parameters,
				double *UpperBounds,
				double *LowerBounds,
				int     NumberOfParameters,
				double  ControlParameter)

/* bounds are not checked ! */
{
 int i;
 double x1,x2, RandomFraction, Sum, *Direction, StepSize;

 CALLOC1D(Direction,NumberOfParameters);

if (ControlParameter > 10)
{
  StepSize = ControlParameter / (double)10;
}else
{
  StepSize = 1;
}

 Sum=0.0;
 for(i=0;i<NumberOfParameters;i++) {
   SA_StandardGauss0(&x1,&x2,&RandomFraction);
   Direction[i] = x1;
   Sum += x1*x1;
 //  printf("Sum %f direction %f \n",Sum, x1);
 }

//create new parameters 

 for(i=0;i<NumberOfParameters;i++) 
 {
   double newValue = Parameters[i] + StepSize * Direction[i];   

   if (newValue < 0)
   {
      ParametersNew[i] = Parameters[i] - StepSize * Direction[i];
   }else
      ParametersNew[i] = newValue;   
 }

//the sum of percentages should be 100
//so lets normalize the new percentages

//the first two parameters
double zero = ParametersNew[0];
double first = ParametersNew[1];

double onePersent = (zero + first) / 100;
ParametersNew[0] = zero / onePersent;
ParametersNew[1] = first / onePersent;

//parameters for three concurent transitions
zero = ParametersNew[2];
first = ParametersNew[3];
double third = ParametersNew[4];

onePersent = (zero + first + third) / 100;
ParametersNew[2] = zero / onePersent;
ParametersNew[3] = first / onePersent;
ParametersNew[4] = third / onePersent;

//next two
zero = ParametersNew[5];
first = ParametersNew[6];

onePersent = (zero + first) / 100;
ParametersNew[5] = zero / onePersent;
ParametersNew[6] = first / onePersent;

//
zero = ParametersNew[7];
first = ParametersNew[8];

onePersent = (zero + first) / 100;
ParametersNew[7] = zero / onePersent;
ParametersNew[8] = first / onePersent;

//
zero = ParametersNew[9];
first = ParametersNew[10];

onePersent = (zero + first) / 100;
ParametersNew[9] = zero / onePersent;
ParametersNew[10] = first / onePersent;

//
zero = ParametersNew[11];
first = ParametersNew[12];

onePersent = (zero + first) / 100;
ParametersNew[11] = zero / onePersent;
ParametersNew[12] = first / onePersent;

//
zero = ParametersNew[13];
first = ParametersNew[14];

onePersent = (zero + first) / 100;
ParametersNew[13] = zero / onePersent;
ParametersNew[14] = first / onePersent;

//
zero = ParametersNew[15];
first = ParametersNew[16];

onePersent = (zero + first) / 100;
ParametersNew[15] = zero / onePersent;
ParametersNew[16] = first / onePersent;

//
zero = ParametersNew[17];
first = ParametersNew[18];

onePersent = (zero + first) / 100;
ParametersNew[17] = zero / onePersent;
ParametersNew[18] = first / onePersent;

//
zero = ParametersNew[19];
first = ParametersNew[20];

onePersent = (zero + first) / 100;
ParametersNew[19] = zero / onePersent;
ParametersNew[20] = first / onePersent;

 FREE1D(Direction);
} 

