/********************************************************************************
 *  Builder.cpp                                                                 *
 *                                                                              *
 *                                                                              *
 *  Petri Nets simulation program                                               *
 *  is written in C++  and it is a part of the PhD research of                  *
 *  Polina Reshetova performed in 2010-2014 at                                  *
 *  Biosystems data analysis group, SILS,                                       *
 *  University van Amsterdam                                                    *
 *                                                                              *
 *  Written by Polina Reshetova                                                 *
 *                                                                              *
 *                                                                              *
 *  This is free software: you can redistribute it and/or modify                *
 *  it under the terms of the GNU Lesser General Public License as published by *
 *  the Free Software Foundation, either version 3 of the License, or           *
 *  (at your option) any later version.                                         *
 *                                                                              *
 *  the software is distributed in the hope that it will be useful,             *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *  GNU Lesser General Public License for more details.                         *
 *                                                                              *
 *  You should have received a copy of the GNU Lesser General Public License    *
 *  along with this software.  If not, see <http://www.gnu.org/licenses/>.      *
 ********************************************************************************/



#include "Builder.h"

Builder::Builder() {
	// TODO Auto-generated constructor stub

}

Builder::~Builder() {
	// TODO Auto-generated destructor stub
}



#include <sys/stat.h>
bool exists(string strFilename) {
  struct stat stFileInfo;
  // Attempt to get the file attributes
  int intStat = stat(strFilename.c_str(),&stFileInfo);
  return (intStat == 0);
}



void print_indentation(unsigned int indentation)
{
  for(unsigned int i = 0; i < indentation; ++i)
    std::cout << " ";
}

void print_node(const xmlpp::Node* node, unsigned int indentation = 0)
{
  std::cout << std::endl; //Separate nodes by an empty line.

  const xmlpp::ContentNode* nodeContent = dynamic_cast<const xmlpp::ContentNode*>(node);
  const xmlpp::TextNode* nodeText = dynamic_cast<const xmlpp::TextNode*>(node);
  const xmlpp::CommentNode* nodeComment = dynamic_cast<const xmlpp::CommentNode*>(node);

  if(nodeText && nodeText->is_white_space()) //Let's ignore the indenting - you don't always want to do this.
    return;

  Glib::ustring nodename = node->get_name();

  if(!nodeText && !nodeComment && !nodename.empty()) //Let's not say "name: text".
  {
    print_indentation(indentation);
    std::cout << "Node name = " << node->get_name() << std::endl;
    std::cout << "Node name = " << nodename << std::endl;
  }
  else if(nodeText) //Let's say when it's text. - e.g. let's say what that white space is.
  {
    print_indentation(indentation);
    std::cout << "Text Node" << std::endl;
  }

  //Treat the various node types differently:
  if(nodeText)
  {
    print_indentation(indentation);
    std::cout << "text = \"" << nodeText->get_content() << "\"" << std::endl;
  }
  else if(nodeComment)
  {
    print_indentation(indentation);
    std::cout << "comment = " << nodeComment->get_content() << std::endl;
  }
  else if(nodeContent)
  {
    print_indentation(indentation);
    std::cout << "content = " << nodeContent->get_content() << std::endl;
  }
  else if(const xmlpp::Element* nodeElement = dynamic_cast<const xmlpp::Element*>(node))
  {
    //A normal Element node:

    //line() works only for ElementNodes.
    print_indentation(indentation);
    std::cout << "     line = " << node->get_line() << std::endl;

    //Print attributes:
    const xmlpp::Element::AttributeList& attributes = nodeElement->get_attributes();
    for(xmlpp::Element::AttributeList::const_iterator iter = attributes.begin(); iter != attributes.end(); ++iter)
    {
      const xmlpp::Attribute* attribute = *iter;
      print_indentation(indentation);
      std::cout << "  Attribute " << attribute->get_name() << " = " << attribute->get_value() << std::endl;
    }

    const xmlpp::Attribute* attribute = nodeElement->get_attribute("title");
    if(attribute)
    {
      std::cout << "title found: =" << attribute->get_value() << std::endl;

    }
  }

  if(!nodeContent)
  {


    xmlpp::Node::NodeList list = node->get_children();

    for(xmlpp::Node::NodeList::iterator iter = list.begin(); iter != list.end(); ++iter)
    {
    	print_node(*iter, indentation + 2); //recursive

    }
  }
}




Net* Builder::buildNet(string filename, Randomizer *randomizer){


	Net* net = new Net();

	/*check the file*/
	if (!exists(filename))
		throw new Exception("File not found");

	xmlpp::Document *doc;
	xmlpp::DomParser parser;

	try
	{
	   //parser.set_validate();
	   parser.set_substitute_entities(); //We just want the text to be resolved/unescaped automatically.
	   parser.parse_file(filename);
	   if(parser)
	   {
		 doc = parser.get_document();
	   }

	 }catch(const std::exception& ex)
	 {
	   std::cout << "Exception caught: " << ex.what() << std::endl;
	 }

	xmlpp::Node* root = doc->get_root_node();

	Glib::ustring speciesXpath = "//sbml/model/listOfSpecies/species";
	xmlpp::NodeSet speciesNodes = root->find(speciesXpath);

	for(xmlpp::NodeSet::iterator iter = speciesNodes.begin(); iter != speciesNodes.end(); ++iter)
    {
		const xmlpp::Element* nodeElement = dynamic_cast<const xmlpp::Element*>(*iter);
		const xmlpp::Attribute* attribute_id = nodeElement->get_attribute("id");

		std::string name;
		int amount = 0;

	    if(attribute_id)
	    {
	      name = attribute_id->get_value();
	    }

	    const xmlpp::Attribute* attribute_amount = nodeElement->get_attribute("initialAmount");
	    if(attribute_amount)
		{
		  Glib::ustring att_amount = attribute_amount->get_value();
		  std::stringstream s;

		  s << att_amount.raw();
		  s >> amount;
		}

	    Place *p = new Place();
	    p->setName(name);
	   // std::cout << "name " << name << "\n" << std::endl;
	    p->setTokensNumber(amount);

	    net->add(p);
    }

	Glib::ustring tblockXpath = "//sbml/model/Tblocks/tblock";
	xmlpp::NodeSet tblocksNodes = root->find(tblockXpath);

	for(xmlpp::NodeSet::iterator iter = tblocksNodes.begin(); iter != tblocksNodes.end(); ++iter)
	{
		const xmlpp::Element* nodeElement = dynamic_cast<const xmlpp::Element*>(*iter);

		TBlock *tb = new TBlock(randomizer);

		Glib::ustring reactionsXpath = "reaction";
		xmlpp::NodeSet reactionsNodes = nodeElement->find(reactionsXpath);

		for(xmlpp::NodeSet::iterator iter = reactionsNodes.begin(); iter != reactionsNodes.end(); ++iter)
	    {
			Transition * t = parseReaction(iter,net);
			tb->add(t);
	    }

		net->add(tb);
	}

	Glib::ustring reactionsXpath = "//sbml/model/listOfReactions/reaction";
	xmlpp::NodeSet reactionsNodes = root->find(reactionsXpath);

	for(xmlpp::NodeSet::iterator iter = reactionsNodes.begin(); iter != reactionsNodes.end(); ++iter)
    {
		Transition *t = parseReaction(iter, net);

		TBlock *tb = new TBlock(randomizer);
		tb->add(t);
		net->add(tb);
    }

	return net;
}


Transition *Builder::parseReaction(xmlpp::NodeSet::iterator iter, Net* net){

	        const xmlpp::Element* nodeElement = dynamic_cast<const xmlpp::Element*>(*iter);
			const xmlpp::Attribute* attribute_id = nodeElement->get_attribute("id");

	    	std::string name;
		    if(attribute_id)
	        {
		     // std::cout << "id: =" << attribute_id->get_value() << std::endl;
		      name = attribute_id->get_value();
		    }

		    Transition *t = new Transition();
		    t->setName(name);

		    int p_int = 0;
		    const xmlpp::Attribute* pvalue_attribute = nodeElement->get_attribute("pvalue");
		    if(pvalue_attribute){
		        Glib::ustring pvalue = pvalue_attribute->get_value();

		        std::stringstream s;

		        s << pvalue.raw();
		        s >> p_int;
		    }

		    t->setProbability(p_int);

		    Glib::ustring reactantsXpath = "listOfReactants/speciesReference";
		    xmlpp::NodeSet input_nodes = nodeElement->find(reactantsXpath);

		    for(xmlpp::NodeSet::iterator reactant_iter = input_nodes.begin(); reactant_iter != input_nodes.end(); ++reactant_iter)
		    {
		    	const xmlpp::Element* nodeElement = dynamic_cast<const xmlpp::Element*>(*reactant_iter);
		    	const xmlpp::Attribute* species_id = nodeElement->get_attribute("species");
		    	Glib::ustring place_name = species_id->get_value();

		    	Place *place = net->findPlaceByName(place_name);
		    	if (place)
		    	{
		    		Edge *edge = new Edge;
		    		edge->setPlace(place);
		    		t->setIncomingEdge(edge);
		    	}
		    }

		    Glib::ustring productsXpath = "listOfProducts/speciesReference";
    	    xmlpp::NodeSet output_nodes = nodeElement->find(productsXpath);

    	    for(xmlpp::NodeSet::iterator reactant_iter = output_nodes.begin(); reactant_iter != output_nodes.end(); ++reactant_iter)
    	    {
    	    	const xmlpp::Element* nodeElement = dynamic_cast<const xmlpp::Element*>(*reactant_iter);
    	    	const xmlpp::Attribute* species_id = nodeElement->get_attribute("species");

    	    	Glib::ustring place_name = species_id->get_value();

    	    	Place *place = net->findPlaceByName(place_name);
    	    	if (place)
    	    	{
    	    		Edge *edge = new Edge;
    	    		edge->setPlace(place);
		    		t->setOutcomingEdge(edge);
	   	    	}
	   	    }

	   	    return t;

}

Place* Builder::handle_place(const xmlpp::Node*)
{
	Place* place = new Place;
	return place;
}

Transition* Builder::handle_transition(const xmlpp::Node*)
{
	Transition* transition = new Transition;
	return transition;
}



//DOM document parser
//DOMDocument* Builder::parse(string inFile){
xmlpp::Document* Builder::parse(string inFile){


	//DEBUG("NetIO.parse start");
	//this->initializeParser();
	//DOMDocument *doc = NULL;

	xmlpp::Node *root;
	xmlpp::Document *doc;
	xmlpp::DomParser parser;
	//then we ask for parser->get_document it returns a link
	//the object is still in the parser
	//if we destroy the parser the document will be destroyed as well
	//so parser and doc have to be in on name space
 try
 {

   //parser.set_validate();
   parser.set_substitute_entities(); //We just want the text to be resolved/unescaped automatically.
   parser.parse_file(inFile);
   if(parser)
   {

	 doc = parser.get_document();
   }


 }catch(const std::exception& ex)
 {
   std::cout << "Exception caught: " << ex.what() << std::endl;
 }

 return doc;
}
