/********************************************************************************
 *                                                                              *
 *  Petri Nets simulation program                                               *
 *  is written in C++  and it is a part of the PhD research of                  *
 *  Polina Reshetova performed in 2010-2014 at                                  *
 *  Biosystems data analysis group, SILS,                                       *
 *  University van Amsterdam                                                    *
 *                                                                              *
 *  Written by Polina Reshetova                                                 *
 *                                                                              *
 *                                                                              *
 *  This is free software: you can redistribute it and/or modify                *
 *  it under the terms of the GNU Lesser General Public License as published by *
 *  the Free Software Foundation, either version 3 of the License, or           *
 *  (at your option) any later version.                                         *
 *                                                                              *
 *  the software is distributed in the hope that it will be useful,             *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *  GNU Lesser General Public License for more details.                         *
 *                                                                              *
 *  You should have received a copy of the GNU Lesser General Public License    *
 *  along with this software.  If not, see <http://www.gnu.org/licenses/>.      *
 ********************************************************************************/


#include <iostream>
#include "net/Net.h"
#include "builder/Builder.h"
#include "simulator/Simulator.h"
#include "Randomizer.h"

int main(int argc, char *argv[]) {
	int c;

	//file with a Petri net model
	char* netFile = "src/samples/6compartments.xml";
	char* stepProfileFile = "stepProfile.txt";

	Builder* builder = new Builder();

	//read parameters from the command line
	char *p,*t;
	while ((c = getopt(argc, argv, "t:p:")) != -1) {
		switch(c){
		case 't':
	    	t = optarg;
	       // std::cout << "t is set " << t << "\n" << std::endl;
	        break;
		case 'p':
	    	p = optarg;
	    //	std::cout << "p is set " << p << "\n" << std::endl;
	        break;
		case '?':
	        std::cout << "unknown arg " << optopt << "\n" << std::endl;
	        break;
	}}

	vector<std::string> transitions;
	vector<int> probabilities;

	char *tr_buf = strtok(t, ":");
	while (tr_buf)
	{
	   std::string tr_string =  tr_buf;
	   transitions.push_back(tr_string);
	   tr_buf = strtok (NULL, ":");
	}

	int j, num;
	char *pEnd, *pch, *linep;
	for (j = 0, linep = p; (pch = strtok(linep, ":")) != NULL;j++, linep = NULL)
	  {
	    num = (int) strtol(pch, &pEnd, 10);

	    probabilities.push_back(num);
	   // std::cout << "probability " << pch << "  " << num << std::endl;
	  }

	//std::cout << transitions.size() << std::endl;

	if (transitions.size() != probabilities.size())
	{
		std::cout << "there must be the same number of transitions and probabilities in the input" << std::endl;

		return 0;
	}

	try{
		Randomizer * r = new Randomizer();
		Net *net =  builder->buildNet(netFile, r);

		//add the parameters to the tree
		net->setProbabilities(transitions, probabilities);

		//net->print();
		Simulator* simulator = new Simulator(net, r);
		simulator->run();

		simulator->printTokensProfiles(stepProfileFile);
	}catch (Exception *e){
		std::cout << e->getMessage() << std::endl;
	}

	return 0;

}
