/********************************************************************************
 *  Net.h                                                                       *
 *                                                                              *
 *                                                                              *
 *  Petri Nets simulation program                                               *
 *  is written in C++  and it is a part of the PhD research of                  *
 *  Polina Reshetova performed in 2010-2014 at                                  *
 *  Biosystems data analysis group, SILS,                                       *
 *  University van Amsterdam                                                    *
 *                                                                              *
 *  Written by Polina Reshetova                                                 *
 *                                                                              *
 *                                                                              *
 *  This is free software: you can redistribute it and/or modify                *
 *  it under the terms of the GNU Lesser General Public License as published by *
 *  the Free Software Foundation, either version 3 of the License, or           *
 *  (at your option) any later version.                                         *
 *                                                                              *
 *  the software is distributed in the hope that it will be useful,             *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *  GNU Lesser General Public License for more details.                         *
 *                                                                              *
 *  You should have received a copy of the GNU Lesser General Public License    *
 *  along with this software.  If not, see <http://www.gnu.org/licenses/>.      *
 ********************************************************************************/


#ifndef NET_H_
#define NET_H_

#include <vector>
#include "Transition.h"
#include "Place.h"
#include "TBlock.h"
#include <string>
#include <cstring>

#include <iostream>

using std::vector;

class Net {
private:
	std::string *id;
	vector<Place*> *places;
	vector<TBlock*> *tblocks;
public:
	Net();
	virtual ~Net();

	bool add(Place *p);
	bool add(TBlock *t);

    Place *findPlaceById(const std::string& id);
    TBlock *findTBlockById(const std::string& id);

    Place *findPlaceByName(const std::string& name);
    TBlock *findTBlockByName(const std::string& name);

	vector<TBlock*> getEnabledTBlocks();

	vector<Place*> *getPlaces();
	vector<Transition*> *getTransitions();
	void print();
	void consumeNewlyProduced();

	void setProbabilities(vector<std::string> transitions, vector<int> probabilities);

};

#endif /* NET_H_ */
