/********************************************************************************
 *  TBlock.cpp                                                                  *
 *                                                                              *
 *                                                                              *
 *  Petri Nets simulation program                                               *
 *  is written in C++  and it is a part of the PhD research of                  *
 *  Polina Reshetova performed in 2010-2014 at                                  *
 *  Biosystems data analysis group, SILS,                                       *
 *  University van Amsterdam                                                    *
 *                                                                              *
 *  Written by Polina Reshetova                                                 *
 *                                                                              *
 *                                                                              *
 *  This is free software: you can redistribute it and/or modify                *
 *  it under the terms of the GNU Lesser General Public License as published by *
 *  the Free Software Foundation, either version 3 of the License, or           *
 *  (at your option) any later version.                                         *
 *                                                                              *
 *  the software is distributed in the hope that it will be useful,             *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *  GNU Lesser General Public License for more details.                         *
 *                                                                              *
 *  You should have received a copy of the GNU Lesser General Public License    *
 *  along with this software.  If not, see <http://www.gnu.org/licenses/>.      *
 ********************************************************************************/


#include "TBlock.h"

TBlock::TBlock(Randomizer * r) {
	// TODO Auto-generated constructor stub
	randomizer = r;
}

TBlock::~TBlock() {
	// TODO Auto-generated destructor stub
}

bool TBlock::isEnabled() const {
	if(transitions.size() > 0)
		return transitions[0]->isEnabled();

	return false;
}

void TBlock::fire() {
	if(transitions.size() == 1){
		Transition *t = transitions[0];
		t->fire();
	}
	else if (transitions.size()>1)
	{
		//p is from 0 to 99 (included)
		int p = rand()%100;

		int start_probability = 0;

		for(int j = 0; j < transitions.size(); j++)
		{
			int tp = transitions[j]->getProbability();
			//std::cout << "the probability: " << tp << std::endl;
			if ((start_probability < p) and (p <= (tp + start_probability)))
			{
				//this is the winner!
				transitions[j]->fire();
				break;
			}else
			{
				start_probability += tp;
			}
		}
	}
}

void TBlock::setName(std::string block_name)
{
	name = block_name;
}

std::string TBlock::getName() const
{
	return name;
}

std::string *TBlock::getId() const
{
	return id;
}

vector<Transition*> *TBlock::getTransitions()
{
	return &transitions;
}

bool TBlock::add(Transition *t)
{
	Transition *tt = findTransitionByName((t->getName()));
	if (tt != NULL)
	{
		return false;
	}

	transitions.push_back(t);
	return true;
}

Transition *TBlock::findTransitionByName(const std::string& name)
{
	for (vector<Transition*>::const_iterator iter = transitions.begin(); iter != transitions.end(); iter++)
	{
		Transition* t = *iter;
		if ((t->getName()) == name)
		{
			return t;
		}
	}
	return NULL;
}

int TBlock::getMaximumTokensNumber()
{
	int max = 0;

	for (vector<Transition*>::const_iterator iter = transitions.begin(); iter != transitions.end(); iter++)
	{
		Transition* t = *iter;
		if ((t->getMaximumTokens()) > max)
		{
			max = t->getMaximumTokens();
		}
	}
	return max;
}
