/********************************************************************************
 *  Transition.cpp                                                              *
 *                                                                              *
 *                                                                              *
 *  Petri Nets simulation program                                               *
 *  is written in C++  and it is a part of the PhD research of                  *
 *  Polina Reshetova performed in 2010-2014 at                                  *
 *  Biosystems data analysis group, SILS,                                       *
 *  University van Amsterdam                                                    *
 *                                                                              *
 *  Written by Polina Reshetova                                                 *
 *                                                                              *
 *                                                                              *
 *  This is free software: you can redistribute it and/or modify                *
 *  it under the terms of the GNU Lesser General Public License as published by *
 *  the Free Software Foundation, either version 3 of the License, or           *
 *  (at your option) any later version.                                         *
 *                                                                              *
 *  the software is distributed in the hope that it will be useful,             *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *  GNU Lesser General Public License for more details.                         *
 *                                                                              *
 *  You should have received a copy of the GNU Lesser General Public License    *
 *  along with this software.  If not, see <http://www.gnu.org/licenses/>.      *
 ********************************************************************************/



#include "Transition.h"

Transition::Transition() {
	// TODO Auto-generated constructor stub

	firedNumber = 0;
}

Transition::~Transition() {
	// TODO Auto-generated destructor stub
}

bool Transition::isEnabled() const
{
	return (maxEnabled()>0);
}

int Transition::maxEnabled() const
{
	int times = -1;

	for (vector<Edge*>::const_iterator iter = incomingEdges.begin(); iter != incomingEdges.end() && times != 0; iter++)
	{
		Edge *e = *iter;
		int v = e->getPlace()->getTokensNumber();
		if (v < times || times < 0)
			times = v;
	}
	//std::cout << times << std::endl;
	return times;
}

void Transition::setIncomingEdge(Edge *edge){
	incomingEdges.push_back(edge);
}

vector<Edge*> *Transition::getIncomingEdges()
{
	return &incomingEdges;
}

vector<Edge*> *Transition::getOutcomingEdges()
{
	return &outcomingEdges;
}

void Transition::setOutcomingEdge(Edge *edge){
	outcomingEdges.push_back(edge);
	//return true;
}

void Transition::setName(std::string transition_name)
{
	name = transition_name;
}

std::string Transition::getName() const
{
	return name;
}

std::string *Transition::getId() const
{
	return id;
}

int Transition::getProbability()
{
	return probability;
}

void Transition::setProbability(int p)
{
	probability = p;
}

bool Transition::fire(){
	if(!isEnabled())
	{
	    return false;
	}

	for (vector<Edge*>::const_iterator iter = incomingEdges.begin(); iter != incomingEdges.end(); iter++)
	{
		Edge *e = *iter;
		e->getPlace()->consume();
	}

	for (vector<Edge*>::iterator iter = outcomingEdges.begin(); iter != outcomingEdges.end(); iter++)
	{
		Edge *e = *iter;
		e->getPlace()->produce();
	}

	firedNumber ++;
	return true;
}

int Transition::getFiredNumber()
{
	return firedNumber;
}

int Transition::getMaximumTokens()
{
	int max = 0;

	for (vector<Edge*>::const_iterator iter = incomingEdges.begin(); iter != incomingEdges.end(); iter++)
	{
		Edge *e = *iter;
		if(e->getPlace()->getTokensNumber() > max)
			max = e->getPlace()->getTokensNumber();
	}

	return max;
}
