/********************************************************************************
 *  Transition.h                                                              *
 *                                                                              *
 *                                                                              *
 *  Petri Nets simulation program                                               *
 *  is written in C++  and it is a part of the PhD research of                  *
 *  Polina Reshetova performed in 2010-2014 at                                  *
 *  Biosystems data analysis group, SILS,                                       *
 *  University van Amsterdam                                                    *
 *                                                                              *
 *  Written by Polina Reshetova                                                 *
 *                                                                              *
 *                                                                              *
 *  This is free software: you can redistribute it and/or modify                *
 *  it under the terms of the GNU Lesser General Public License as published by *
 *  the Free Software Foundation, either version 3 of the License, or           *
 *  (at your option) any later version.                                         *
 *                                                                              *
 *  the software is distributed in the hope that it will be useful,             *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *  GNU Lesser General Public License for more details.                         *
 *                                                                              *
 *  You should have received a copy of the GNU Lesser General Public License    *
 *  along with this software.  If not, see <http://www.gnu.org/licenses/>.      *
 ********************************************************************************/


#include "Edge.h"
#include <vector>

using std::vector;

#ifndef TRANSITION_H_
#define TRANSITION_H_

class Transition {
private:
	std::string name;
	std::string *id;

	//probability to fire
	int probability;
	//number of times the transition fired
	int firedNumber;

	vector<Edge*> incomingEdges;
	vector<Edge*> outcomingEdges;

	int maxEnabled() const;

public:
	Transition();
	virtual ~Transition();

	std::string getName() const;
	void setName(std::string transition_name);

    std::string *getId() const;

	bool isEnabled() const;

	void setIncomingEdge(Edge *edge);
	void setOutcomingEdge(Edge *edge);

	vector<Edge*> *getIncomingEdges();
	vector<Edge*> *getOutcomingEdges();

	bool fire();

	int getProbability();
	void setProbability(int p);

	int getMaximumTokens();

	int getFiredNumber();

};

#endif /* TRANSITION_H_ */
