/********************************************************************************
 *                                                                              *
 *  Profiler.cpp                                                                *
 *                                                                              *
 *                                                                              *
 *  Petri Nets simulation program                                               *
 *  is written in C++  and it is a part of the PhD research of                  *
 *  Polina Reshetova performed in 2010-2014 at                                  *
 *  Biosystems data analysis group, SILS,                                       *
 *  University van Amsterdam                                                    *
 *                                                                              *
 *  Written by Polina Reshetova                                                 *
 *                                                                              *
 *                                                                              *
 *  This is free software: you can redistribute it and/or modify                *
 *  it under the terms of the GNU Lesser General Public License as published by *
 *  the Free Software Foundation, either version 3 of the License, or           *
 *  (at your option) any later version.                                         *
 *                                                                              *
 *  the software is distributed in the hope that it will be useful,             *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *  GNU Lesser General Public License for more details.                         *
 *                                                                              *
 *  You should have received a copy of the GNU Lesser General Public License    *
 *  along with this software.  If not, see <http://www.gnu.org/licenses/>.      *
 ********************************************************************************/



#include "Profiler.h"

Profiler::Profiler(Net * net_ref) {
	// TODO Auto-generated constructor stub
	net = net_ref;
}

Profiler::~Profiler() {
	// TODO Auto-generated destructor stub
}

void Profiler::record()
{
	//place_numbers is vector with place names
	//concentrations is a vector of vectors.
	//first vector contains places the second contains concentrations for each place
	//serial number of a place in the place_numbers is the same as in the concentrations

	//std::cout << net->getTransitions()->size() << std::endl;
	vector<Place*> *places = net->getPlaces();

	vector <int> step_conc (places->size());

	for (vector<Place*>::const_iterator iter = places->begin(); iter != places->end(); iter++)
	{
		Place *p = *iter;
		std::string name = p->getName();

		//save all names in the vector place_numers
		int number = 0;
		for (int i = 0; i < place_numbers.size(); i++)
		{
			if (place_numbers[i] == name)
			{
				number = i+1;
			}
		}

		if(number == 0)
	    {
				place_numbers.push_back(name);
				number = place_numbers.size();
		}

		number--;
		int tokens_number = p->getTokensNumber();

		step_conc[number] = tokens_number;
	}

	concentrations.push_back(step_conc);

}

void Profiler::printTokensProfiles(char *logFile){
		 std::ofstream myfile;
		 myfile.open (logFile);
		 //myfile << "Writing this to a file.\n";

		 //save only genistein profile
		 //find genistein place number
		 int agl_blood_id = 0;
		 int glyc_blood_id = 0;
		 int liver_conjigates_id = 0;
		 int genistein_id = 0;
		 //genistein_lumen_id is 0! because genistein_lumen is the first one!
		for (int i = 0; i < place_numbers.size(); i++)
		{
			//put to the file
			if (place_numbers[i] == "genistein_epithelium")
			{
				genistein_id = i;
			}
			if (place_numbers[i] == "genistein_liver")
			{
				agl_blood_id = i;
			}
			if (place_numbers[i] == "glucuronid_liver")
			{
				glyc_blood_id = i;
			}
//			if (place_numbers[i] == "sulphate_vein")
//		    {
//				liver_conjigates_id = i;
//			}
		}

		if (genistein_id == 0 or agl_blood_id == 0 or glyc_blood_id == 0 )
		{
			std::cout << "one id is missing in the profiler" << std::endl;
			std::cout << genistein_id << "\t" << agl_blood_id<< "\t"<<glyc_blood_id<< std::endl;
		}

		//number of steps
		int steps_number = concentrations.size();

		//number of steps in one hour
		int steps_in_hour = steps_number / 36;

		int time_points[7] = {0,3,5,9,16,24,36};
		int t = 0;

		for (int i = 0; i < 7; i++)
		{
				int step = time_points[i]*steps_in_hour;
				if (i!=0)
				{
					step -= 1;
				}
			//	std::cout << "i is "<< i<<" step is " << step << std::endl;

				myfile << concentrations[step][genistein_id] << "\t" << concentrations[step][agl_blood_id] << "\t" << concentrations[step][glyc_blood_id]  << "\n";// << concentrations[step][liver_conjigates_id] << "\n";
		}

        myfile.close();
}

void Profiler::printAll(char *logFile)
{
	//fix for small input number (as 10 tokens)
	if (concentrations.size() < 36)
	{
		vector <int> step_conc (place_numbers.size());
		for (int j = 0; j < place_numbers.size(); j++)
		{
			step_conc[j] = 0;
		}

		while(concentrations.size() < 36)
		{
		//	std::cout << "consentration size "<< concentrations.size() << std::endl;
			concentrations.push_back(step_conc);
		}
	}

	//number of steps
	int steps_number = concentrations.size();

	//number of steps in one hour
	int steps_in_hour = steps_number / 36;
	//	std::cout << "steps number " << steps_number << std::endl;
	//	std::cout << "step in hour " << steps_in_hour << std::endl;

	int time_points[] = {0,3,5,9,16,24,36};

	std::ofstream myfile;
	myfile.open (logFile);

	for (int i = 0; i < 7; i++)
	{
		int step = time_points[i]*steps_in_hour;
		if (i!=0)
		{
			step -= 1;
		}
	//	std::cout << "i is "<< i<<" step is " << step << std::endl;
		for (int j = 0; j < place_numbers.size(); j++)
		{
			myfile << concentrations[step][j] << "\t";
		}
		myfile << "\n";
	}

    myfile.close();
}

void Profiler::printTimeProfiles(int coeff, int no_use, char* logFile, vector<Place*> places)
{
	std::ofstream myfile;
	myfile.open (logFile);

	//coeff is how many time points in total
	//then how many steps in one time point is
	int step_in_tp = concentrations.size() / coeff;
	std::cout << " steps in time point " << concentrations.size() << "  "<< step_in_tp << std::endl;

	//coeff+2 to have enough space for residuals from concentrations.size() - step_in_tp * coeff
	//and to have first step in 0 time point
	//so two extra time points are needed - for 0 step and for the residuals
	int size_of_shrink = coeff+2;
	vector< vector <int> > shrink (size_of_shrink,vector <int> (place_numbers.size(),0));

	for (int i = 0; i < place_numbers.size(); i++)
	{
		//put to the file
		myfile << "place title: " << place_numbers[i] << "\t";
	}
	myfile << "\n";

	int step = 0;
	for (int i = 0; i < concentrations.size(); i++)
	{
		//i is the step
		//if it is coeff step, write it
		if (i % step_in_tp == 0)
		{

			for (int j = 0; j < place_numbers.size(); j++)
			{
				shrink[step][j] += concentrations[i][j];

				myfile << shrink[step][j] << "\t";
			}

			step ++;
			myfile << "\n";
		}else
		{
			for (int j = 0; j < concentrations[i].size(); j++)
		    {
				shrink[step][j] += concentrations[i][j];
		    }
		}
	}
	myfile.close();
}

void Profiler::printFlowProfiles(Net* net, char* logFile)
{
	std::ofstream myfile;
    //open the file
	myfile.open (logFile);

	vector<Transition*> *transitions = net->getTransitions();

	for (vector<Transition*>::const_iterator iter = transitions->begin(); iter != transitions->end(); iter++)
	{
		Transition* t = *iter;

		myfile << t->getName() << "\t" << t->getFiredNumber() << "\n";
	}

	myfile.close();
}
