/********************************************************************************
 *                                                                              *
 *  Simulator.cpp                                                               *
 *                                                                              *
 *                                                                              *
 *  Petri Nets simulation program                                               *
 *  is written in C++  and it is a part of the PhD research of                  *
 *  Polina Reshetova performed in 2010-2014 at                                  *
 *  Biosystems data analysis group, SILS,                                       *
 *  University van Amsterdam                                                    *
 *                                                                              *
 *  Written by Polina Reshetova                                                 *
 *                                                                              *
 *                                                                              *
 *  This is free software: you can redistribute it and/or modify                *
 *  it under the terms of the GNU Lesser General Public License as published by *
 *  the Free Software Foundation, either version 3 of the License, or           *
 *  (at your option) any later version.                                         *
 *                                                                              *
 *  the software is distributed in the hope that it will be useful,             *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *  GNU Lesser General Public License for more details.                         *
 *                                                                              *
 *  You should have received a copy of the GNU Lesser General Public License    *
 *  along with this software.  If not, see <http://www.gnu.org/licenses/>.      *
 ********************************************************************************/

#include "Simulator.h"

Simulator::Simulator(Net *net_ref, Randomizer * r) {
	// TODO Auto-generated constructor stub
	net = net_ref;
	randomizer = r;
	profiler = new Profiler(net_ref);

	currentStep = 0;
	maxSteps = 4000000;//should be bigger then 36. see Profiler steps_number and steps_in_hour

}



Simulator::~Simulator() {
	// TODO Auto-generated destructor stub
}

void Simulator::run(){
	//it is in for loop and not in recursion because recursion might use all CPU memory
	//already on 200000 steps for some Petri nets models

	for (int i_step=0; i_step<maxSteps; i_step++)
	{
	 // std::cout << "current step " << currentStep << std::endl;
	  vector<TBlock*> tblocks = net->getEnabledTBlocks();

	  if (tblocks.size() > 0)
	  {
		//save the current stage of the net
		profiler->record();

		//make a step, consume tokens
		step(&tblocks);

		//and SAVE newly produced tokens
	    net->consumeNewlyProduced();

	    if (currentStep < maxSteps)
	    {
	    	currentStep++ ;
	    }
	  }else
	  {
		break;
	  }
	}
}



void Simulator::printTokensProfiles(char* logFile)
{
	profiler->printTokensProfiles(logFile);
	//profiler->printAll(logFile);
}

void Simulator::printTimeProfiles(int coeff, int sim_number,char* logFile, vector<Place*> places)
{
	profiler->printTimeProfiles(coeff,sim_number, logFile, places);
}

void Simulator::printFlowProfiles(Net *net, char* logFile)
{
	profiler->printFlowProfiles(net, logFile);
}

void Simulator::step(vector<TBlock*> *tblocks)
{
	//p is from 0 to 99 (included)

	int p = randomizer->getNumber();
	//std::cout << "p is " << p << std::endl;

	int start_probability = 0;
	int all_tokens = 0;

	for (vector<TBlock*>::const_iterator i = tblocks->begin(); i!= tblocks->end(); i++)
	{
		TBlock* tb = *i;
		all_tokens += tb->getMaximumTokensNumber();
	}

	for (vector<TBlock*>::const_iterator i = tblocks->begin(); i!= tblocks->end(); i++)
	{
		TBlock* tb = *i;
		int tb_tokens = tb->getMaximumTokensNumber();

	//	std::cout << "Maximum tokens " << tb_tokens  <<  std::endl;

		int tp = 0;
		if (tb_tokens == all_tokens)
		{
			tp = 100;
		}else{
			float tp_ostatok = float(tb_tokens) / float(all_tokens);
			tp = tp_ostatok * 100;
		}

		if ((start_probability <= p) and (p < (tp + start_probability)))
		{
				//this is the winner!
				tb->fire();
				break;
		}else
		{
				start_probability += tp;
		}
	}
}
