/********************************************************************************
 *                                                                              *
 *  Simulator.h                                                               *
 *                                                                              *
 *                                                                              *
 *  Petri Nets simulation program                                               *
 *  is written in C++  and it is a part of the PhD research of                  *
 *  Polina Reshetova performed in 2010-2014 at                                  *
 *  Biosystems data analysis group, SILS,                                       *
 *  University van Amsterdam                                                    *
 *                                                                              *
 *  Written by Polina Reshetova                                                 *
 *                                                                              *
 *                                                                              *
 *  This is free software: you can redistribute it and/or modify                *
 *  it under the terms of the GNU Lesser General Public License as published by *
 *  the Free Software Foundation, either version 3 of the License, or           *
 *  (at your option) any later version.                                         *
 *                                                                              *
 *  the software is distributed in the hope that it will be useful,             *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *  GNU Lesser General Public License for more details.                         *
 *                                                                              *
 *  You should have received a copy of the GNU Lesser General Public License    *
 *  along with this software.  If not, see <http://www.gnu.org/licenses/>.      *
 ********************************************************************************/


#ifndef SIMULATOR_H_
#define SIMULATOR_H_

#include "../net/Net.h"
#include "../net/TBlock.h"

#include "../profiler/Profiler.h"

#include <iostream>;
#include <vector>;
#include <sys/time.h>

#include "../Randomizer.h";

class Simulator {
private:
	Net * net;
	Randomizer * randomizer;

protected:
	void step(vector<TBlock*> *tblocks);

	int maxSteps;
	int currentStep;
	Profiler *profiler;

public:
	Simulator();
	Simulator(Net * net, Randomizer *r);
	virtual ~Simulator();
	void run();

	void printTokensProfiles(char* logFile);
	void printTimeProfiles(int coeff, int sim_number, char* logFile, vector<Place*>);
	void printFlowProfiles(Net *net, char* logFile);
};

#endif /* SIMULATOR_H_ */
